@@ -7,11 +7,55 @@ import re
 from datetime import datetime
 from time import sleep
 from logging import getLogger, INFO, basicConfig
+from requests.exceptions import ConnectTimeout, ConnectionError
+from urllib3.exceptions import ProtocolError
+from http.client import RemoteDisconnected
 
 OBJECT_ID_RE = r".*\/md\/.*\/obj\/(\d+)"
 COOLDOWN_SECONDS = 20
 
 
+class RetrySession():
+    def __init__(self, logger = None):
+        self.session = requests.Session()
+        if logger is None:
+            self.logger = getLogger("RetrySession")
+        else:
+            self.logger = logger
+    def get(self, *args, **kwargs):
+        retry_count = 0
+        retry_timeout = 10
+        while True:
+            try:
+                return self.session.get(*args, **kwargs)
+            except (ConnectionError, ConnectTimeout, RemoteDisconnected, ProtocolError) as error:
+                self.logger.error(error)
+                retry_count += 1
+                if retry_count > 5:
+                    self.logger.error("Too many retries")
+                    raise error
+                if retry_count == 3:
+                    retry_timeout = 30
+                self.logger.error("Retry # %d in %d seconds", retry_count, retry_timeout)
+                sleep(retry_timeout)
+    def post(self, *args, **kwargs):
+        retry_count = 0
+        retry_timeout = 10
+        while True:
+            try:
+                return self.session.post(*args, **kwargs)
+            except (ConnectionError, ConnectTimeout, RemoteDisconnected, ProtocolError) as error:
+                self.logger.error(error)
+                retry_count += 1
+                if retry_count > 5:
+                    self.logger.error("Too many retries")
+                    raise error
+                if retry_count == 3:
+                    retry_timeout = 30
+                self.logger.error("Retry # %d in %d seconds", retry_count, retry_timeout)
+                sleep(retry_timeout)
+
+
 def get_object_id(obj_link):
     """Return object id from object link
 
@@ -29,7 +73,7 @@ def _url(domain, endpoint):
 
 
 class ExportJob:
-    def __init__(self, token, status_uri, gd, done=False):
+    def __init__(self, token, status_uri, gd, done=False, exported_object=None):
         """ExportJob constructor.
 
         Args:
@@ -37,9 +81,11 @@ class ExportJob:
             status_uri (str): Uri to poll status
             gd (GoodData): GoodData connection object that created the export job
             done (bool, optional): Whether the job is complete. Defaults to False.
+            exported_object (str, optional): Exported object's title for better logging.
         """
         self.token = token
         self.status_uri = status_uri
+        self.exported_objects = exported_object
         self.done = done
         self.gd = gd
 
@@ -68,6 +114,8 @@ class ExportJob:
         if response.json()["wTaskStatus"]["status"] == "OK":
             self.done = True
             return True
+        if response.json()["wTaskStatus"]["status"] == "ERROR":
+            raise Exception(response.text)
         return False
 
 
@@ -85,7 +133,7 @@ class GoodData:
             nocache (bool, optional): If True, discard any possible existing cache. Defaults to False.
         """
         self.last_login = None
-        self.session = requests.session()
+        self.session = RetrySession()
         self.user = user
         self.password = password
         self.domain = domain
@@ -288,6 +336,19 @@ class GoodData:
         with open(fname, "w", encoding="utf-8") as fid:
             fid.write(response.text)
 
+    def get_user(self, user_link: str):
+        """Get user details. `user_link` may be either whole link (starting /gdc/account/profile) or user id
+
+        Args:
+            user_link (str): User link or user id
+        """
+        self.login()
+        if "/gdc/account/profile" in user_link:
+            url = _url(self.domain, user_link[1:])
+        else:
+            url = _url(self.domain, "gdc/account/profile/{}".format(user_link))
+        return self.session.get(url).json()
+
     def list_projects(self, nocache=False):
         """List all projects authenticated by current user.
 
@@ -523,7 +584,7 @@ class GoodData:
         }
         response = self.session.post(url, headers=self.headers, data=json.dumps(body))
         job_info = response.json()["partialMDArtifact"]
-        return ExportJob(job_info["token"], job_info["status"]["uri"], self)
+        return ExportJob(job_info["token"], job_info["status"]["uri"], self, exported_object=object_id)
 
     def partial_export_sync(self, object_id):
         """Create partial export job and wait for it to complete.
